classdef RTreeStyle < handle

  ## -*- texinfo -*-
  ## @deftp {Class} RTreeStyle
  ##
  ## A Ruby wrapper around RTree styles, used in PostScript plotting.
  ##
  ## @end deftp
  ##
  ## @deftypefn {RTreeStyle} {@var{obj} =} RTreeStyle ()
  ##
  ## Creates an empty @code{RTreeStyle} object, one can subsequently
  ## call the instance method @code{json}
  ##
  ## @seealso{RTreeStyle.json_read, RTree.postscript}
  ##
  ## @end deftypefn

  properties (Access = private)
    state
  endproperties

  methods (Static = true)

    function obj = RTreeStyle()
      # nada
    endfunction

    ## -*- texinfo -*-
    ## @deftypefn {RTreeStyle} {@var{obj} =} json_read (@var{io_arg})
    ##
    ## Create a new RTreeStyle instance @var{obj} from JSON stream
    ## or path @var{io_arg}.
    ##
    ## @end deftypefn

    function obj = json_read(io_arg)
      if nargin != 1
        error('RTree:args-number', 'FILE required');
      endif
      obj.state = RTreeStyle.io_read('json_read', io_arg, 'r');
    endfunction

    ## -*- texinfo -*-
    ## @deftypefn {RTreeStyle} {@var{obj} =} from_json (@var{json})
    ##
    ## Create a new RTreeStyle instance @var{obj} from JSON string.
    ##
    ## @end deftypefn

    function obj = from_json(json)
      if nargin != 1
        error('RTree:args-number', 'JSON required');
      endif
      obj.state = RTreeStyle_dispatch('from_json', json);
    endfunction

  endmethods

  methods (Static = true, Private = true)

    function val = io_read(method, io_arg, mode, varargin)
      if is_valid_file_id(io_arg)
        val = RTreeStyle_dispatch(method, io_arg, varargin{:});
      elseif ischar(io_arg)
        io = fopen(io_arg, mode);
        try
          val = RTreeStyle_dispatch(method, io, varargin{:});
        catch
          fclose(io);
          rethrow(lasterror);
        end_try_catch
        fclose(io);
      else
        error('RTree:args-type', 'expected string or file descriptor');
      endif
    endfunction

  endmethods

endclassdef
